@@ -31,6 +31,7 @@ import Queue
 import pickle
 import datetime
 import math
+import urllib2
 
 import xml.sax.saxutils
 
@@ -43,12 +44,15 @@ from icecast_handler import FeedIcecast
 from shoutcast_handler import FeedShoutcast
 from shoutcast_handler import ShoutcastRadioStation
 from board_handler import FeedBoard
+from board_handler import BoardHandler
+from radiotime_handler import FeedRadioTime
+from radiotime_handler import FeedRadioTimeLocal
 
 #TODO: should not be defined here, but I don't know where to get it from. HELP: much apreciated
 RB_METADATA_FIELD_TITLE = 0
 RB_METADATA_FIELD_GENRE = 4
 RB_METADATA_FIELD_BITRATE = 20
-BOARD_ROOT = "http://segler.bplaced.net/"
+BOARD_ROOT = "http://www.radio-browser.info/"
 RECENTLY_USED_FILENAME = "recently2.bin"
 BOOKMARKS_FILENAME = "bookmarks.bin"
 
@@ -76,12 +80,12 @@ class RadioBrowserSource(rb.StreamingSource):
 				progress = min (float(self.load_current_size) / self.load_total_size, 1.0)
 			return (self.load_status,None,progress)
 		else:
-			return (_("Nothing"),None,0.0)
+			return (_("Nothing"),None,2.0)
 
 	def update_download_status(self,filename,current, total):
 		self.load_current_size = current
 		self.load_total_size = total
-		self.load_status = _("Loading")+" : "+filename
+		self.load_status = _("Loading %(url)s") % {'url':filename}
 
 		gtk.gdk.threads_enter()
 		self.notify_status_changed()
@@ -164,11 +168,13 @@ class RadioBrowserSource(rb.StreamingSource):
 			column_bitrate.set_fixed_width(100)
 			self.tree_view.append_column(column_bitrate)"""
 
+			self.info_box_tree = gtk.HBox()
+
 			# add some more listeners for tree view...
 			# - row double click
 			self.tree_view.connect("row-activated",self.row_activated_handler)
 			# - selection change
-			self.tree_view.connect("cursor-changed",self.treeview_cursor_changed_handler)
+			self.tree_view.connect("cursor-changed",self.treeview_cursor_changed_handler,self.info_box_tree)
 
 			# create icon view
 			self.icon_view = gtk.IconView()
@@ -201,17 +207,49 @@ class RadioBrowserSource(rb.StreamingSource):
 			filterbox.pack_start(gtk.Label(_("Bitrate")+":"),False)
 			filterbox.pack_start(self.filter_entry_bitrate,False)
 
-			self.info_box = gtk.HBox()
+			self.start_box = gtk.HPaned()
+
+			# prepare search tab
+			self.info_box_search = gtk.HBox()
+			self.search_box = gtk.VBox()
+			self.search_entry = gtk.Entry()
+			#self.search_entry.connect("changed",self.filter_entry_changed)
+
+			def searchButtonClick(widget):
+				self.doSearch(self.search_entry.get_text())
+			searchbutton = gtk.Button(_("Search"))
+			searchbutton.connect("clicked",searchButtonClick)
+
+			search_input_box = gtk.HBox()
+			search_input_box.pack_start(self.search_entry)
+			search_input_box.pack_start(searchbutton,False)
+
+			self.result_box = gtk.TreeView()
+			self.result_box.connect("row-activated",self.row_activated_handler)
+			self.result_box.connect("cursor-changed",self.treeview_cursor_changed_handler,self.info_box_search)
+
+			self.result_box_container = gtk.ScrolledWindow()
+			self.result_box_container.set_shadow_type(gtk.SHADOW_IN)
+			self.result_box_container.add(self.result_box)
+			self.result_box_container.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
+			self.result_box.append_column(gtk.TreeViewColumn(_("Title"),gtk.CellRendererText(),text=0))
+
+			self.search_box.pack_start(search_input_box,False)
+			self.search_box.pack_start(self.result_box_container)
+			self.search_box.pack_start(self.info_box_search,False)
 
 			stations_box = gtk.VBox()
 			stations_box.pack_start(filterbox,False)
 			stations_box.pack_start(self.view)
-			stations_box.pack_start(self.info_box,False)
+			stations_box.pack_start(self.info_box_tree,False)
 
 			self.notebook = gtk.Notebook()
+			self.notebook.append_page(self.start_box,gtk.Label(_("Favourites")))
+			self.notebook.append_page(self.search_box,gtk.Label(_("Search")))
 			self.notebook.append_page(stations_box,gtk.Label(_("Radiostation list")))
 			self.notebook.set_scrollable(True)
-
+			self.notebook.connect("switch-page",self.event_page_switch)
+			
 			self.pack_start(self.notebook)
 			self.notebook.show_all()
 			self.icon_view_container.hide_all()
@@ -229,10 +267,309 @@ class RadioBrowserSource(rb.StreamingSource):
 			self.icon_download_thread.start()
 
 			# first time filling of the model
-			self.refill_list()
+			self.main_list_filled = False
+			
+			# enable images on buttons
+			settings = gtk.settings_get_default()
+			settings.set_property("gtk-button-images",True)
 
 		rb.BrowserSource.do_impl_activate (self)
 
+	def searchEngines(self):
+		yield FeedLocal(self.cache_dir,self.update_download_status)
+		yield FeedIcecast(self.cache_dir,self.update_download_status)
+		yield FeedBoard(self.cache_dir,self.update_download_status)
+		yield FeedShoutcast(self.cache_dir,self.update_download_status)
+		yield FeedRadioTime(self.cache_dir,self.update_download_status)
+
+	def doSearch(self, term):
+		search_model = gtk.ListStore(str)
+		search_model.append((_("Searching for : '%s'") % term,))
+
+		# unset model
+		self.result_box.set_model(search_model)
+		# start thread
+		search_thread = threading.Thread(target = self.doSearchThread, args = (term,))
+		search_thread.start()
+
+	def doSearchThread(self,term):
+		results = {}
+		self.station_actions = {}
+
+		# check each engine for search method
+		for feed in self.searchEngines():
+			try:
+				feed.search
+			except:
+				print "no search support in : "+feed.name()
+				continue
+
+			# call search method
+			try:
+				self.station_actions[feed.name()] = feed.get_station_actions()
+				result = feed.search(term)
+				results[feed.name()] = result
+			except Exception,e:
+				print "error with source:"+feed.name()
+				print "error:"+str(e)
+
+		gtk.gdk.threads_enter()
+		# create new model
+		new_model = gtk.TreeStore(str,object)
+		# add entries to model
+		for name in results.keys():
+			result = results[name]
+			source_parent = new_model.append(None,(name+" ("+str(len(result))+")",None))
+			for entry in result:
+				new_model.append(source_parent,(entry.server_name,entry))
+
+		# set model of result_box
+		new_model.set_sort_column_id(0,gtk.SORT_ASCENDING)
+		self.result_box.set_model(new_model)
+		gtk.gdk.threads_leave()
+			
+	def download_click_statistic(self):
+		# download statistics
+		statisticsStr = ""
+		try:
+			remotefile = urllib2.urlopen("http://www.radio-browser.info/topclick.php?limit=10")
+			statisticsStr = remotefile.read()
+		
+		except Exception, e:
+			print "download failed exception"
+			print e
+			return
+		
+		# parse statistics
+		self.statistics_handler = BoardHandler()
+		xml.sax.parseString(statisticsStr,self.statistics_handler)
+	
+		# fill statistics box
+		self.refill_statistics(thread=True)
+	
+	def shortStr(self,longstring,maxlen):
+		if len(longstring) > maxlen:
+			short_value = longstring[0:maxlen-3]+"..."
+		else:
+			short_value = longstring
+		return short_value
+	
+	def refill_statistics(self, thread=False):
+		# check if already downloaded
+		try:
+			self.statistics_handler
+		except:
+			transmit_thread = threading.Thread(target = self.download_click_statistic)
+			transmit_thread.start()
+			return
+		
+		def button_click(widget,name,station):
+			self.play_uri(station)
+		
+		def button_add_click(widget,name,station):
+			data = self.load_from_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME))
+			if data is None:
+				data = {}
+			if station.server_name not in data:
+				data[station.server_name] = station
+			self.save_to_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME),data)
+			
+			self.refill_favourites()
+		
+		if thread:
+			gtk.gdk.threads_enter()
+		for entry in self.statistics_handler.entries:
+			button = gtk.Button(self.shortStr(entry.server_name,30)+" ("+entry.clickcount+")")
+			button.connect("clicked",button_click,entry.server_name,entry)
+			
+			button_add = gtk.Button()
+			img = gtk.Image()
+			img.set_from_stock(gtk.STOCK_GO_FORWARD,gtk.ICON_SIZE_BUTTON)
+			button_add.set_image(img)
+			button_add.connect("clicked",button_add_click,entry.server_name,entry)
+			line = gtk.HBox()
+			line.pack_start(button)
+			line.pack_start(button_add,expand=False)
+			
+			self.statistics_box.pack_start(line,expand=False)
+			line.show_all()
+			
+		self.statistics_box_parent.show_all()
+		if thread:
+			gtk.gdk.threads_leave()
+
+	def refill_favourites(self):
+		print "refill favourites"
+		width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_BUTTON)
+		
+		# remove all old information in infobox
+		for widget in self.start_box.get_children():
+			self.start_box.remove(widget)
+		
+		def button_click(widget,name,station):
+			self.play_uri(station)
+		
+		def button_record_click(widget,name,station):
+			self.record_uri(station)
+		
+		def button_add_click(widget,name,station):
+			data = self.load_from_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME))
+			if data is None:
+				data = {}
+			if station.server_name not in data:
+				data[station.server_name] = station
+			self.save_to_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME),data)
+			
+			self.refill_favourites()
+		
+		def button_delete_click(widget,name,station):
+			data = self.load_from_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME))
+			if data is None:
+				data = {}
+			if station.server_name in data:
+				del data[station.server_name]
+			self.save_to_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME),data)
+			
+			self.refill_favourites()
+		
+		left_box = gtk.VBox()
+		left_box.show()
+		
+		# add click statistics list
+		self.statistics_box = gtk.VBox()
+		scrolled_box = gtk.ScrolledWindow()
+		scrolled_box.add_with_viewport(self.statistics_box)
+		scrolled_box.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
+		decorated_box = gtk.Frame(_("Click statistics (Last 30 days)"))
+		decorated_box.add(scrolled_box)
+		self.statistics_box_parent = decorated_box
+		left_box.pack_start(decorated_box)
+		
+		self.refill_statistics()
+		
+		# add recently played list
+		recently_box = gtk.VBox()
+		scrolled_box = gtk.ScrolledWindow()
+		scrolled_box.add_with_viewport(recently_box)
+		scrolled_box.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
+		decorated_box = gtk.Frame(_("Recently played"))
+		decorated_box.add(scrolled_box)
+		left_box.pack_start(decorated_box)
+		
+		self.start_box.pack1(left_box)
+		
+		data = self.load_from_file(os.path.join(self.cache_dir,RECENTLY_USED_FILENAME))
+		if data is None:
+			data = {}
+		dataNew = {}
+		sortedkeys = sorted(data.keys())
+		for name in sortedkeys:
+			station = data[name]
+			if datetime.datetime.now()-station.PlayTime <= datetime.timedelta(days=float(self.plugin.recently_played_purge_days)):
+				if len(name) > 53:
+					short_value = name[0:50]+"..."
+				else:
+					short_value = name
+				button = gtk.Button(short_value)
+				button.connect("clicked",button_click,name,station)
+				
+				button_add = gtk.Button()
+				img = gtk.Image()
+				img.set_from_stock(gtk.STOCK_GO_FORWARD,gtk.ICON_SIZE_BUTTON)
+				button_add.set_image(img)
+				button_add.connect("clicked",button_add_click,name,station)
+				line = gtk.HBox()
+				line.pack_start(button)
+				line.pack_start(button_add,expand=False)
+				
+				recently_box.pack_start(line, expand=False)
+				dataNew[name] = station
+				
+				try:
+					if station.icon_src != "":
+						hash_src = hashlib.md5(station.icon_src).hexdigest()
+						filepath = os.path.join(self.icon_cache_dir, hash_src)
+						if os.path.exists(filepath):
+							buffer = gtk.gdk.pixbuf_new_from_file_at_size(filepath,width,height)
+							img = gtk.Image()
+							img.set_from_pixbuf(buffer)
+							img.show()
+							button.set_image(img)
+				except:
+					print "could not set image for station:"+str(station.server_name)
+		
+		if len(sortedkeys)>0:
+			decorated_box.show_all()
+		self.save_to_file(os.path.join(self.cache_dir,RECENTLY_USED_FILENAME),dataNew)
+
+		# add bookmarks
+		favourites_box = gtk.VBox()
+		scrolled_box = gtk.ScrolledWindow()
+		scrolled_box.add_with_viewport(favourites_box)
+		scrolled_box.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
+		decorated_box = gtk.Frame(_("Favourites"))
+		decorated_box.add(scrolled_box)
+		self.start_box.pack2(decorated_box)
+		
+		data = self.load_from_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME))
+		if data is None:
+			data = {}
+		sortedkeys = sorted(data.keys())
+		for name in sortedkeys:
+			line = gtk.HBox()
+			station = data[name]
+			if len(name) > 53:
+				short_value = name[0:50]+"..."
+			else:
+				short_value = name
+			
+			button = gtk.Button(short_value)
+			button.connect("clicked",button_click,name,station)
+			button_delete = gtk.Button()
+			img = gtk.Image()
+			img.set_from_stock(gtk.STOCK_DELETE,gtk.ICON_SIZE_BUTTON)
+			button_delete.set_image(img)
+			button_delete.connect("clicked",button_delete_click,name,station)
+			
+			button_record = gtk.Button()
+			img = gtk.Image()
+			img.set_from_stock(gtk.STOCK_MEDIA_RECORD,gtk.ICON_SIZE_BUTTON)
+			button_record.set_image(img)
+			button_record.connect("clicked",button_record_click,name,station)
+			
+			line.pack_start(button)
+			line.pack_start(button_record,expand=False)
+			line.pack_start(button_delete,expand=False)
+			favourites_box.pack_start(line, expand=False)
+			
+			try:
+				if station.icon_src != "":
+					hash_src = hashlib.md5(station.icon_src).hexdigest()
+					filepath = os.path.join(self.icon_cache_dir, hash_src)
+					if os.path.exists(filepath):
+						buffer = gtk.gdk.pixbuf_new_from_file_at_size(filepath,width,height)
+						img = gtk.Image()
+						img.set_from_pixbuf(buffer)
+						img.show()
+						button.set_image(img)
+			except:
+				print "could not set image for station:"+str(station.server_name)
+		if (len(sortedkeys) > 0):
+			decorated_box.show_all()
+
+	""" handler for page switches in the main notebook """
+	def event_page_switch(self,notebook,page,page_num):
+		if page_num == 0:
+			# update favourites each time user selects it
+			self.refill_favourites()
+		if page_num == 1:
+			pass
+		if page_num == 2:
+			if not self.main_list_filled:
+				# fill the list only the first time, the user selects the main tab
+				self.main_list_filled = True
+				self.refill_list()
+
 	""" listener on double click in search view """
 	def on_item_activated_icon_view(self,widget,item):
 		model = widget.get_model()
@@ -250,20 +587,20 @@ class RadioBrowserSource(rb.StreamingSource):
 			self.update_info_box(obj)
 	
 	""" listener for selection changes """
-	def treeview_cursor_changed_handler(self,treeview):
+	def treeview_cursor_changed_handler(self,treeview,info_box):
 		# get selected item
-		selection = self.tree_view.get_selection()
+		selection = treeview.get_selection()
 		model,iter = selection.get_selected()
 
 		# if some item is selected
 		if not iter == None:
 			obj = model.get_value(iter,1)
-			self.update_info_box(obj)
+			self.update_info_box(obj,info_box)
 
-	def update_info_box(self,obj):
+	def update_info_box(self,obj,info_box):
 		# remove all old information in infobox
-		for widget in self.info_box.get_children():
-			self.info_box.remove(widget)
+		for widget in info_box.get_children():
+			info_box.remove(widget)
 
 		# create new infobox
 		info_container = gtk.Table(12,2)
@@ -272,6 +609,8 @@ class RadioBrowserSource(rb.StreamingSource):
 
 		# convinience method for adding new labels to infobox
 		def add_label(title,value,shorten=True):
+			if value == None:
+				return
 			if not value == "":
 				if shorten:
 					if len(value) > 53:
@@ -337,7 +676,6 @@ class RadioBrowserSource(rb.StreamingSource):
 
 		def button_play_handler(widget,station):
 			self.play_uri(station)
-			pass
 
 		def button_bookmark_handler(widget,station):
 			data = self.load_from_file(os.path.join(self.cache_dir,BOOKMARKS_FILENAME))
@@ -370,7 +708,6 @@ class RadioBrowserSource(rb.StreamingSource):
 			transmit_thread = threading.Thread(target = self.download_feed,args = (feed,))
 			transmit_thread.setDaemon(True)
 			transmit_thread.start()
-			pass
 
 		def button_action_handler(widget,action):
 			action.call(self)
@@ -431,11 +768,11 @@ class RadioBrowserSource(rb.StreamingSource):
 		sub_info_box.pack_start(info_container)
 		sub_info_box.pack_start(button_box,False)
 
-		decorated_info_box = gtk.Frame("Info box")
+		decorated_info_box = gtk.Frame(_("Info box"))
 		decorated_info_box.add(sub_info_box)
 
-		self.info_box.pack_start(decorated_info_box)
-		self.info_box.show_all()
+		info_box.pack_start(decorated_info_box)
+		info_box.show_all()
 
 	""" icon download worker thread function """
 	def icon_download_worker(self):
@@ -631,6 +968,7 @@ class RadioBrowserSource(rb.StreamingSource):
 
 	""" starts playback of the station """
 	def play_uri(self,station):
+		station.updateRealURL()
 		play_thread = threading.Thread(target = self.play_uri_,args = (station,))
 		play_thread.setDaemon(True)
 		play_thread.start()
@@ -647,7 +985,7 @@ class RadioBrowserSource(rb.StreamingSource):
 			tryno += 1
 
 			gtk.gdk.threads_enter()
-			self.load_status = _("downloading station information")+" '"+station.server_name+"', "+_("Try")+":"+str(tryno)+"/"+str(math.trunc(float(self.plugin.download_trys)))
+			self.load_status = _("downloading station information")+" '"+station.server_name+"', "+_("Try")+":"+str(tryno)+"/"+str(math.floor(float(self.plugin.download_trys)))
 			self.load_total_size = 0
 			self.notify_status_changed()
 			gtk.gdk.threads_leave()
@@ -672,10 +1010,16 @@ class RadioBrowserSource(rb.StreamingSource):
 			if data is None:
 				data = {}
 			if station.server_name not in data:
-				self.tree_store.append(self.recently_iter,(station.server_name,station))
+				try:
+					self.tree_store.append(self.recently_iter,(station.server_name,station))
+				except:
+					pass
 				data[station.server_name] = station
 				data[station.server_name].PlayTime = datetime.datetime.now()
-				self.save_to_file(os.path.join(self.cache_dir,RECENTLY_USED_FILENAME),data)
+			else:
+				data[station.server_name].PlayTime = datetime.datetime.now()
+			
+			self.save_to_file(os.path.join(self.cache_dir,RECENTLY_USED_FILENAME),data)
 
 		gtk.gdk.threads_enter()
 		self.load_status = ""
@@ -716,7 +1060,6 @@ class RadioBrowserSource(rb.StreamingSource):
 			#shell.load_uri(uri,False)
 
 			# start playback
-			player.play()
 			player.play_entry(self.entry,self)
 
 		gtk.gdk.threads_leave()
@@ -732,6 +1075,8 @@ class RadioBrowserSource(rb.StreamingSource):
 		myiter = model.get_iter(path)
 		
 		obj = model.get_value(myiter,1)
+		if obj == None:
+			return
 
 		if isinstance(obj,RadioStation):
 			station = obj
@@ -751,7 +1096,7 @@ class RadioBrowserSource(rb.StreamingSource):
 			tryno += 1
 
 			gtk.gdk.threads_enter()
-			self.load_status = _("downloading feed")+" '"+feed.name()+"' "+_("from")+" '"+feed.uri+"', "+_("Try")+":"+str(tryno)+"/"+str(math.trunc(float(self.plugin.download_trys)))
+			self.load_status = _("Downloading feed %(name)s from %(url)s. %(try)d/%(trys)d") % {'name':feed.name(), 'url':feed.uri, 'try':tryno, 'trys':(math.floor(float(self.plugin.download_trys)))}
 			self.load_total_size = 0
 			self.notify_status_changed()
 			gtk.gdk.threads_leave()
@@ -774,15 +1119,19 @@ class RadioBrowserSource(rb.StreamingSource):
 		self.refill_list()
 
 	def do_impl_delete_thyself(self):
-		# kill all running records
-		for uri in self.recording_streams.keys():
-			self.recording_streams[uri].stop()
+		if self.hasActivated:
+			# kill all running records
+			for uri in self.recording_streams.keys():
+				self.recording_streams[uri].stop()
+			self.shell = False
 
 	def engines(self):
 		yield FeedLocal(self.cache_dir,self.update_download_status)
 		yield FeedIcecast(self.cache_dir,self.update_download_status)
 		yield FeedBoard(self.cache_dir,self.update_download_status)
 		yield FeedShoutcast(self.cache_dir,self.update_download_status)
+		yield FeedRadioTime(self.cache_dir,self.update_download_status)
+		yield FeedRadioTimeLocal(self.cache_dir,self.update_download_status)
 
 	def get_stock_icon(self, name):
 		theme = gtk.icon_theme_get_default()
@@ -820,7 +1169,7 @@ class RadioBrowserSource(rb.StreamingSource):
 		local_icon = self.load_icon_file(self.plugin.find_file("local-logo.png"),None)
 
 		gtk.gdk.threads_enter()
-		self.load_status = _("loading feed")+" '"+feed.name()+"'"
+		self.load_status = _("Loading feed %(name)s") % {'name':feed.name(),}
 		self.load_total_size = 0
 		self.notify_status_changed()
 		gtk.gdk.threads_leave()
@@ -839,7 +1188,7 @@ class RadioBrowserSource(rb.StreamingSource):
 		entries = feed.entries()
 
 		gtk.gdk.threads_enter()
-		self.load_status = _("integrating feed")+" '"+feed.name()+"'("+str(len(entries))+" items)"+_("into tree")+"..."
+		self.load_status = _("Integrating feed %(name)s (%(itemcount)d items) into tree...") % {'name':feed.name(),'itemcount':len(entries)}
 		self.notify_status_changed()
 		gtk.gdk.threads_leave()
 
